@@ -1,25 +1,27 @@
 import random
 from collections import defaultdict
-from typing import Optional, Tuple
+from typing import Optional, Tuple, Sequence
 
 import tensorflow as tf
-from tensorflow_similarity.types import Tensor
+from tensorflow_similarity.types import FloatTensor, IntTensor, Tensor
 from tqdm.auto import tqdm
 
 from .samplers import Augmenter, Sampler
+from .utils import select_examples
 
 
 class MultiShotMemorySampler(Sampler):
-
+    # FIXME: typing x, y is hard.
     def __init__(self,
-                 x: Tensor,
-                 y: Tensor,
-                 class_per_batch: int,
-                 example_per_class: int = 2,
+                 x,
+                 y,
+                 classes_per_batch: int = 2,
+                 examples_per_class_per_batch: int = 2,
                  steps_per_epoch: int = 1000,
+                 class_list: Sequence[int] = None,
+                 total_examples_per_class: int = None,
                  augmenter: Optional[Augmenter] = None,
                  warmup: int = -1):
-
         """Create a Multishot in memory sampler that ensures that each batch is
         well balanced. That is, each batch aims to contain
         `example_per_class` examples of `class_per_batch`
@@ -47,14 +49,23 @@ class MultiShotMemorySampler(Sampler):
 
             y: labels.
 
-            class_per_batch: Numbers of class to include in a single batch
+            class_per_batch: Numbers of distinct class to include in a
+            single batch
 
-            example_per_class: How many example of each class to use per batch.
-            Defaults to 2.
+            examples_per_class_per_batch: How many example of each class
+            to use per batch. Defaults to 2.
 
             steps_per_epoch: How many steps/batches per epoch.
             Defaults to 1000.
 
+            class_list: Filter the list of examples to only keep those who
+            belong to the supplied class list.
+
+            total_examples_per_class: Restrict the number of examples for EACH
+            class to total_examples_per_class if set. If not set, all the
+            available examples are selected. Defaults to None - no selection.
+
+
             augmenter: A function that takes a batch in and return a batch out.
             Can alters the number of examples returned which in turn change the
             batch_size used. Defaults to None.
@@ -64,31 +75,46 @@ class MultiShotMemorySampler(Sampler):
             boolean `is_warmup`. See `self.get_examples()` Defaults to 0.
         """
 
-        super().__init__(class_per_batch,
-                         example_per_class=example_per_class,
-                         steps_per_epoch=steps_per_epoch,
-                         augmenter=augmenter,
-                         warmup=warmup)
+        super().__init__(
+            classes_per_batch,
+            examples_per_class_per_batch=examples_per_class_per_batch,  # noqa
+            steps_per_epoch=steps_per_epoch,
+            augmenter=augmenter,
+            warmup=warmup)
+
+        # precompute information we need
+        if not class_list:
+            self.class_list = list(set([int(e) for e in y]))
+        else:
+            # dedup in case user mess up and cast in case its a tensor
+            self.class_list = list(set([int(c) for c in class_list]))
+
+        if classes_per_batch > len(self.class_list):
+            raise ValueError("the value of class_per_batch must be <= to the\
+                                number of existing class in the dataset")
+
+        # filter
+        x, y = select_examples(x,
+                               y,
+                               class_list=self.class_list,
+                               num_examples_per_class=total_examples_per_class)
+
+        # assign after potential selection
         self.x = x
         self.y = y
 
-        # precompute information we need
-        class_list, _ = tf.unique(y)
-        self.class_list = [int(e) for e in class_list]
-
-        # Mapping class to idx
+        # we need to reindex here  as the numbers of samples might have
+        # changed due to the filtering
         # In this sampler, contrary to file based one, the pool of examples
         # wont' change so we can optimize by doing this step in the constructor
         self.index_per_class = defaultdict(list)
+        cls = [int(c) for c in y]  # need to cast as  tensor lookup are slowww
         for idx in tqdm(range(len(x)), desc='indexing classes'):
-            cl = int(y[idx])  # need to cast tensor
+            cl = cls[idx]
             self.index_per_class[cl].append(idx)
 
-    def get_examples(self,
-                     batch_id: int,
-                     num_classes: int,
-                     example_per_class: int
-                     ) -> Tuple[Tensor, Tensor]:
+    def get_examples(self, batch_id: int, num_classes: int,
+                     examples_per_class: int) -> Tuple[Tensor, Tensor]:
 
         # select class at ramdom
         class_list = random.sample(self.class_list, k=num_classes)
@@ -97,7 +123,7 @@ class MultiShotMemorySampler(Sampler):
         idxs = []
         for class_id in class_list:
             class_idxs = self.index_per_class[class_id]
-            idxs.extend(random.choices(class_idxs, k=example_per_class))
+            idxs.extend(random.choices(class_idxs, k=examples_per_class))
 
         random.shuffle(idxs)
         batch_x = tf.gather(self.x, idxs[:self.batch_size])
@@ -113,7 +139,6 @@ class SingleShotMemorySampler(Sampler):
                  class_per_batch: int,
                  steps_per_epoch: int = 1000,
                  warmup: int = -1) -> None:
-
         """Create a single shot in memory sampler.
 
         The `batch_size` used during training will be equal to:
@@ -159,11 +184,8 @@ class SingleShotMemorySampler(Sampler):
         self.x = x
         self.num_elts = len(x)
 
-    def get_examples(self,
-                     batch_id: int,
-                     num_classes: int,
-                     example_per_class: int
-                     ) -> Tuple[Tensor, Tensor]:
+    def get_examples(self, batch_id: int, num_classes: int,
+                     example_per_class: int) -> Tuple[Tensor, Tensor]:
         _ = batch_id
         _ = example_per_class
 
